@@ -1,9 +1,12 @@
 import { testWithFeatureToggles } from 'test/test-utils';
 
+import { config } from '@grafana/runtime';
 import { PromAlertingRuleState, PromRuleGroupDTO, PromRuleType } from 'app/types/unified-alerting-dto';
 
 import { mockGrafanaPromAlertingRule, mockPromRecordingRule } from '../../mocks';
 import { RuleHealth } from '../../search/rulesSearchParser';
+import { pluginMeta, pluginMetaToPluginConfig } from '../../testSetup/plugins';
+import { SupportedPlugin } from '../../types/pluginBridges';
 import { Annotation } from '../../utils/constants';
 import { getDatasourceAPIUid } from '../../utils/datasource';
 import { getFilter } from '../../utils/search';
@@ -257,12 +260,55 @@ describe('grafana-managed rules', () => {
       expect(backendFilter.health).toEqual([]);
       expect(backendFilter.contactPoint).toBeUndefined();
     });
+
+    it('should not set hasInvalidDataSourceNames flag when no data source names are provided', () => {
+      const { hasInvalidDataSourceNames } = getGrafanaFilter(getFilter({}));
+
+      expect(hasInvalidDataSourceNames).toBe(false);
+    });
   });
 
   describe('backend filtering with alertingUIUseBackendFilters feature toggle', () => {
     describe('when alertingUIUseBackendFilters is enabled', () => {
       testWithFeatureToggles({ enable: ['alertingUIUseBackendFilters'] });
 
+      it('should include datasources in backend filter when valid data source names are provided', () => {
+        const { backendFilter, hasInvalidDataSourceNames } = getGrafanaFilter(
+          getFilter({ dataSourceNames: ['prometheus', 'loki'] })
+        );
+
+        expect(backendFilter.datasources).toEqual(['datasource-uid-1', 'datasource-uid-3']);
+        expect(hasInvalidDataSourceNames).toBe(false);
+      });
+
+      it('should detect invalid data source names and set hasInvalidDataSourceNames flag', () => {
+        const { backendFilter, hasInvalidDataSourceNames } = getGrafanaFilter(
+          getFilter({ dataSourceNames: ['non-existent-datasource'] })
+        );
+
+        expect(backendFilter.datasources).toEqual([]);
+        expect(hasInvalidDataSourceNames).toBe(true);
+      });
+
+      it('should include only valid datasource UIDs when some names are invalid', () => {
+        const { backendFilter, hasInvalidDataSourceNames } = getGrafanaFilter(
+          getFilter({ dataSourceNames: ['prometheus', 'non-existent-datasource'] })
+        );
+
+        expect(backendFilter.datasources).toEqual(['datasource-uid-1']);
+        expect(hasInvalidDataSourceNames).toBe(false); // Not all are invalid
+      });
+
+      it('should skip dataSourceNames filtering on frontend when backend filtering is enabled', () => {
+        const rule = mockGrafanaPromAlertingRule({
+          queriedDatasourceUIDs: ['datasource-uid-1'],
+        });
+
+        const { frontendFilter } = getGrafanaFilter(getFilter({ dataSourceNames: ['loki'] }));
+        // Should return true because dataSourceNames filter is null (handled by backend).
+        expect(frontendFilter.ruleMatches(rule)).toBe(true);
+      });
+
       it('should include title in backend filter when freeFormWords are provided', () => {
         const { backendFilter } = getGrafanaFilter(getFilter({ freeFormWords: ['cpu', 'usage'] }));
 
@@ -373,19 +419,60 @@ describe('grafana-managed rules', () => {
         expect(frontendFilter.groupMatches(group)).toBe(true);
       });
 
-      it('should still apply other frontend filters', () => {
-        const rule = mockGrafanaPromAlertingRule({
+      it('should include ruleMatchers in backend filter when labels are provided', () => {
+        const { backendFilter } = getGrafanaFilter(getFilter({ labels: ['severity=critical'] }));
+
+        expect(backendFilter.ruleMatchers).toBeDefined();
+        expect(backendFilter.ruleMatchers).toHaveLength(1);
+        expect(backendFilter.ruleMatchers).toEqual([
+          '{"name":"severity","value":"critical","isRegex":false,"isEqual":true}',
+        ]);
+      });
+
+      it('should include plugins in backend filter and skip frontend filtering', () => {
+        // Set up test plugin as installed
+        config.apps[SupportedPlugin.Slo] = pluginMetaToPluginConfig(pluginMeta[SupportedPlugin.Slo]);
+
+        const regularRule = mockGrafanaPromAlertingRule({
           name: 'High CPU Usage',
           labels: { severity: 'critical', team: 'ops' },
           alerts: [],
         });
 
-        // Label filter should still work on frontend
-        const { frontendFilter } = getGrafanaFilter(getFilter({ labels: ['severity=warning'] }));
-        expect(frontendFilter.ruleMatches(rule)).toBe(false);
+        const pluginRule = mockGrafanaPromAlertingRule({
+          name: 'Plugin Rule',
+          labels: { __grafana_origin: `plugin/${SupportedPlugin.Slo}` },
+          alerts: [],
+        });
 
-        const { frontendFilter: frontendFilter2 } = getGrafanaFilter(getFilter({ labels: ['severity=critical'] }));
-        expect(frontendFilter2.ruleMatches(rule)).toBe(true);
+        // Plugins filter should be handled by backend
+        const { backendFilter, frontendFilter } = getGrafanaFilter(getFilter({ plugins: 'hide' }));
+
+        // Backend filter should include plugins parameter
+        expect(backendFilter.plugins).toBe('hide');
+
+        // Frontend filter should pass through all rules (no filtering)
+        expect(frontendFilter.ruleMatches(regularRule)).toBe(true);
+        expect(frontendFilter.ruleMatches(pluginRule)).toBe(true);
+      });
+
+      it('should include searchFolder in backend filter when namespace is provided', () => {
+        const { backendFilter } = getGrafanaFilter(getFilter({ namespace: 'my-folder' }));
+
+        expect(backendFilter.searchFolder).toBe('my-folder');
+      });
+
+      it('should skip namespace filtering on frontend when backend filtering is enabled', () => {
+        const group: PromRuleGroupDTO = {
+          name: 'Test Group',
+          file: 'production/alerts',
+          rules: [],
+          interval: 60,
+        };
+
+        const { frontendFilter } = getGrafanaFilter(getFilter({ namespace: 'staging' }));
+        // Should return true because namespace filter is null (handled by backend)
+        expect(frontendFilter.groupMatches(group)).toBe(true);
       });
     });
 
@@ -469,6 +556,12 @@ describe('grafana-managed rules', () => {
         expect(backendFilter.searchGroupName).toBeUndefined();
       });
 
+      it('should not include searchFolder in backend filter', () => {
+        const { backendFilter } = getGrafanaFilter(getFilter({ namespace: 'my-folder' }));
+
+        expect(backendFilter.searchFolder).toBeUndefined();
+      });
+
       it('should perform groupName filtering on frontend', () => {
         const group: PromRuleGroupDTO = {
           name: 'CPU Usage Alerts',
@@ -556,25 +649,32 @@ describe('grafana-managed rules', () => {
         const { frontendFilter: groupNoMatch } = getGrafanaFilter(getFilter({ groupName: 'memory' }));
         expect(groupNoMatch.groupMatches(group)).toBe(false);
 
-        // Always-frontend filters (labels, dataSourceNames, namespace) should work
+        // Always-frontend filters (labels, namespace) should work.
         const { frontendFilter: labelsMatch } = getGrafanaFilter(getFilter({ labels: ['severity=critical'] }));
         expect(labelsMatch.ruleMatches(alertingRule)).toBe(true);
 
         const { frontendFilter: labelsNoMatch } = getGrafanaFilter(getFilter({ labels: ['severity=warning'] }));
         expect(labelsNoMatch.ruleMatches(alertingRule)).toBe(false);
 
-        const { frontendFilter: dsMatch } = getGrafanaFilter(getFilter({ dataSourceNames: ['prometheus'] }));
-        expect(dsMatch.ruleMatches(alertingRule)).toBe(true);
-
-        const { frontendFilter: dsNoMatch } = getGrafanaFilter(getFilter({ dataSourceNames: ['loki'] }));
-        expect(dsNoMatch.ruleMatches(alertingRule)).toBe(false);
-
         const { frontendFilter: nsMatch } = getGrafanaFilter(getFilter({ namespace: 'production' }));
         expect(nsMatch.groupMatches(group)).toBe(true);
 
         const { frontendFilter: nsNoMatch } = getGrafanaFilter(getFilter({ namespace: 'staging' }));
         expect(nsNoMatch.groupMatches(group)).toBe(false);
       });
+
+      it('should skip dataSourceNames filtering on frontend (handled by backend)', () => {
+        const alertingRule = mockGrafanaPromAlertingRule({
+          queriedDatasourceUIDs: ['datasource-uid-1'],
+        });
+
+        // DataSourceNames is backend-filtered when feature toggle is enabled.
+        const { frontendFilter: dsMatch } = getGrafanaFilter(getFilter({ dataSourceNames: ['prometheus'] }));
+        expect(dsMatch.ruleMatches(alertingRule)).toBe(true);
+
+        const { frontendFilter: dsNoMatch } = getGrafanaFilter(getFilter({ dataSourceNames: ['loki'] }));
+        expect(dsNoMatch.ruleMatches(alertingRule)).toBe(true);
+      });
     });
 
     describe('when both alertingUIUseBackendFilters and alertingUIUseFullyCompatBackendFilters are enabled', () => {
@@ -631,29 +731,8 @@ describe('grafana-managed rules', () => {
         expect(frontendFilter.groupMatches(group)).toBe(true);
       });
 
-      it('should still apply always-frontend filters (labels, dataSourceNames, namespace)', () => {
-        const rule = mockGrafanaPromAlertingRule({
-          name: 'High CPU Usage',
-          labels: { severity: 'critical' },
-          queriedDatasourceUIDs: ['datasource-uid-1'],
-          alerts: [],
-        });
-
-        // Labels filter should still work
-        const { frontendFilter: labelFilter } = getGrafanaFilter(getFilter({ labels: ['severity=warning'] }));
-        expect(labelFilter.ruleMatches(rule)).toBe(false);
-
-        const { frontendFilter: labelFilter2 } = getGrafanaFilter(getFilter({ labels: ['severity=critical'] }));
-        expect(labelFilter2.ruleMatches(rule)).toBe(true);
-
-        // DataSourceNames filter should still work
-        const { frontendFilter: dsFilter } = getGrafanaFilter(getFilter({ dataSourceNames: ['prometheus'] }));
-        expect(dsFilter.ruleMatches(rule)).toBe(true);
-
-        const { frontendFilter: dsFilter2 } = getGrafanaFilter(getFilter({ dataSourceNames: ['loki'] }));
-        expect(dsFilter2.ruleMatches(rule)).toBe(false);
-
-        // Namespace filter should still work
+      it('should skip namespace filtering on frontend', () => {
+        // Namespace filter should be handled by backend
         const group: PromRuleGroupDTO = {
           name: 'Test Group',
           file: 'production/alerts',
@@ -665,7 +744,20 @@ describe('grafana-managed rules', () => {
         expect(nsFilter.groupMatches(group)).toBe(true);
 
         const { frontendFilter: nsFilter2 } = getGrafanaFilter(getFilter({ namespace: 'staging' }));
-        expect(nsFilter2.groupMatches(group)).toBe(false);
+        expect(nsFilter2.groupMatches(group)).toBe(true);
+      });
+
+      it('should skip dataSourceNames filtering on frontend (handled by backend)', () => {
+        const rule = mockGrafanaPromAlertingRule({
+          queriedDatasourceUIDs: ['datasource-uid-1'],
+        });
+
+        // DataSourceNames is backend-filtered when both feature toggles are enabled.
+        const { frontendFilter: dsFilter } = getGrafanaFilter(getFilter({ dataSourceNames: ['prometheus'] }));
+        expect(dsFilter.ruleMatches(rule)).toBe(true);
+
+        const { frontendFilter: dsFilter2 } = getGrafanaFilter(getFilter({ dataSourceNames: ['loki'] }));
+        expect(dsFilter2.ruleMatches(rule)).toBe(true);
       });
     });
   });
@@ -732,10 +824,20 @@ describe('grafana-managed rules', () => {
         expect(hasGrafanaClientSideFilters(getFilter({ groupName: 'test-group' }))).toBe(false);
       });
 
-      it('should return true for client-side only filters', () => {
-        expect(hasGrafanaClientSideFilters(getFilter({ namespace: 'production' }))).toBe(true);
-        expect(hasGrafanaClientSideFilters(getFilter({ dataSourceNames: ['prometheus'] }))).toBe(true);
-        expect(hasGrafanaClientSideFilters(getFilter({ labels: ['severity=critical'] }))).toBe(true);
+      it('should return false for dataSourceNames (handled by backend when feature toggle is enabled)', () => {
+        expect(hasGrafanaClientSideFilters(getFilter({ dataSourceNames: ['prometheus'] }))).toBe(false);
+      });
+
+      it('should return false for labels (handled by backend when feature toggle is enabled)', () => {
+        expect(hasGrafanaClientSideFilters(getFilter({ labels: ['severity=critical'] }))).toBe(false);
+      });
+
+      it('should return false for namespace filter (handled by backend)', () => {
+        expect(hasGrafanaClientSideFilters(getFilter({ namespace: 'production' }))).toBe(false);
+      });
+
+      it('should return false for plugins filter (handled by backend when feature toggle is enabled)', () => {
+        expect(hasGrafanaClientSideFilters(getFilter({ plugins: 'hide' }))).toBe(false);
       });
 
       it('should return false for backend-only filters (state, health, contactPoint)', () => {
@@ -749,21 +851,24 @@ describe('grafana-managed rules', () => {
       testWithFeatureToggles({ enable: ['alertingUIUseFullyCompatBackendFilters'] });
 
       it('should return correct values for all filter types', () => {
-        // Should return false for: empty, backend-handled (ruleType, dashboardUid), and backend-only filters
+        // Should return false for: empty, backend-handled (ruleType, dashboardUid, dataSourceNames), and backend-only filters
         expect(hasGrafanaClientSideFilters(getFilter({}))).toBe(false);
         expect(hasGrafanaClientSideFilters(getFilter({ ruleType: PromRuleType.Alerting }))).toBe(false);
         expect(hasGrafanaClientSideFilters(getFilter({ dashboardUid: 'test-dashboard' }))).toBe(false);
+        expect(hasGrafanaClientSideFilters(getFilter({ dataSourceNames: ['prometheus'] }))).toBe(false);
         expect(hasGrafanaClientSideFilters(getFilter({ ruleState: PromAlertingRuleState.Firing }))).toBe(false);
         expect(hasGrafanaClientSideFilters(getFilter({ ruleHealth: RuleHealth.Ok }))).toBe(false);
         expect(hasGrafanaClientSideFilters(getFilter({ contactPoint: 'my-contact-point' }))).toBe(false);
 
-        // Should return true for: frontend-handled filters
+        // Should return true for: frontend-handled filters (labels, namespace)
         expect(hasGrafanaClientSideFilters(getFilter({ freeFormWords: ['cpu'] }))).toBe(true);
         expect(hasGrafanaClientSideFilters(getFilter({ ruleName: 'alert' }))).toBe(true);
         expect(hasGrafanaClientSideFilters(getFilter({ groupName: 'test-group' }))).toBe(true);
         expect(hasGrafanaClientSideFilters(getFilter({ namespace: 'production' }))).toBe(true);
-        expect(hasGrafanaClientSideFilters(getFilter({ dataSourceNames: ['prometheus'] }))).toBe(true);
         expect(hasGrafanaClientSideFilters(getFilter({ labels: ['severity=critical'] }))).toBe(true);
+
+        // plugins is backend-handled when alertingUIUseFullyCompatBackendFilters is enabled
+        expect(hasGrafanaClientSideFilters(getFilter({ plugins: 'hide' }))).toBe(false);
       });
     });
 
@@ -782,10 +887,15 @@ describe('grafana-managed rules', () => {
         expect(hasGrafanaClientSideFilters(getFilter({ ruleHealth: RuleHealth.Ok }))).toBe(false);
         expect(hasGrafanaClientSideFilters(getFilter({ contactPoint: 'my-contact-point' }))).toBe(false);
 
-        // Should return true for: always-frontend filters only
-        expect(hasGrafanaClientSideFilters(getFilter({ namespace: 'production' }))).toBe(true);
-        expect(hasGrafanaClientSideFilters(getFilter({ dataSourceNames: ['prometheus'] }))).toBe(true);
-        expect(hasGrafanaClientSideFilters(getFilter({ labels: ['severity=critical'] }))).toBe(true);
+        // Should return false for: namespace (handled by backend)
+        expect(hasGrafanaClientSideFilters(getFilter({ namespace: 'production' }))).toBe(false);
+
+        // plugins is backend-handled when both feature toggles are enabled
+        expect(hasGrafanaClientSideFilters(getFilter({ plugins: 'hide' }))).toBe(false);
+
+        // Should return false for: backend-handled filters when both feature toggles are enabled
+        expect(hasGrafanaClientSideFilters(getFilter({ dataSourceNames: ['prometheus'] }))).toBe(false);
+        expect(hasGrafanaClientSideFilters(getFilter({ labels: ['severity=critical'] }))).toBe(false);
       });
     });
   });
