@@ -198,7 +198,7 @@ describe('DashboardDatasourceBehaviour', () => {
 
       expect(spy).toHaveBeenCalledTimes(1);
       // since there is no previous request ID on dashboard load, the behaviour should not re-run queries
-      expect(behaviour['prevRequestId']).toBeUndefined();
+      expect(behaviour['prevRequestIds'].size).toBe(0);
     });
 
     it('Should not re-run queries in behaviour on scene load', async () => {
@@ -242,7 +242,7 @@ describe('DashboardDatasourceBehaviour', () => {
 
       expect(spy).toHaveBeenCalledTimes(1);
       // since there is no previous request ID on dashboard load, the behaviour should not re-run queries
-      expect(behaviour['prevRequestId']).toBeUndefined();
+      expect(behaviour['prevRequestIds'].size).toBe(0);
     });
 
     it('Should exit behaviour early if not in a dashboard scene', async () => {
@@ -593,9 +593,370 @@ describe('DashboardDatasourceBehaviour', () => {
 
       expect(spy).toHaveBeenCalled();
     });
+
+    it('Should re-run query when ANY source panel changes with multiple dashboardDS queries', async () => {
+      jest.spyOn(console, 'error').mockImplementation();
+
+      // Create two source panels
+      const sourcePanel1 = new VizPanel({
+        title: 'Panel A',
+        pluginId: 'table',
+        key: 'panel-1',
+        $data: new SceneDataTransformer({
+          transformations: [],
+          $data: new SceneQueryRunner({
+            datasource: { uid: 'grafana' },
+            queries: [{ refId: 'A', queryType: 'randomWalk' }],
+          }),
+        }),
+      });
+
+      const sourcePanel2 = new VizPanel({
+        title: 'Panel B',
+        pluginId: 'table',
+        key: 'panel-2',
+        $data: new SceneDataTransformer({
+          transformations: [],
+          $data: new SceneQueryRunner({
+            datasource: { uid: 'grafana' },
+            queries: [{ refId: 'A', queryType: 'randomWalk' }],
+          }),
+        }),
+      });
+
+      // Create a mixed DS panel that references BOTH source panels
+      const mixedDSPanel = new VizPanel({
+        title: 'Panel C - Mixed',
+        pluginId: 'table',
+        key: 'panel-3',
+        $data: new SceneDataTransformer({
+          transformations: [],
+          $data: new SceneQueryRunner({
+            datasource: { uid: MIXED_DATASOURCE_NAME },
+            queries: [
+              {
+                datasource: { uid: SHARED_DASHBOARD_QUERY },
+                refId: 'A',
+                panelId: 1,
+              },
+              {
+                datasource: { uid: SHARED_DASHBOARD_QUERY },
+                refId: 'B',
+                panelId: 2,
+              },
+            ],
+            $behaviors: [new DashboardDatasourceBehaviour({})],
+          }),
+        }),
+      });
+
+      const scene = new DashboardScene({
+        title: 'hello',
+        uid: 'dash-1',
+        meta: {
+          canEdit: true,
+        },
+        body: DefaultGridLayoutManager.fromVizPanels([sourcePanel1, sourcePanel2, mixedDSPanel]),
+      });
+
+      const sceneDeactivate = activateFullSceneTree(scene);
+
+      await new Promise((r) => setTimeout(r, 1));
+
+      const spy = jest
+        .spyOn(mixedDSPanel.state.$data!.state.$data as SceneQueryRunner, 'runQueries')
+        .mockImplementation();
+
+      // deactivate scene
+      sceneDeactivate();
+
+      // Only change the SECOND source panel
+      (sourcePanel2.state.$data!.state.$data as SceneQueryRunner).runQueries();
+
+      await new Promise((r) => setTimeout(r, 1));
+
+      // activate scene again
+      activateFullSceneTree(scene);
+
+      // Should re-run because the second panel changed
+      expect(spy).toHaveBeenCalled();
+    });
+
+    it('Should track multiple dashboardDS queries independently', async () => {
+      jest.spyOn(console, 'error').mockImplementation();
+
+      const sourcePanel1 = new VizPanel({
+        title: 'Panel A',
+        pluginId: 'table',
+        key: 'panel-1',
+        $data: new SceneDataTransformer({
+          transformations: [],
+          $data: new SceneQueryRunner({
+            datasource: { uid: 'grafana' },
+            queries: [{ refId: 'A', queryType: 'randomWalk' }],
+          }),
+        }),
+      });
+
+      const sourcePanel2 = new VizPanel({
+        title: 'Panel B',
+        pluginId: 'table',
+        key: 'panel-2',
+        $data: new SceneDataTransformer({
+          transformations: [],
+          $data: new SceneQueryRunner({
+            datasource: { uid: 'grafana' },
+            queries: [{ refId: 'A', queryType: 'randomWalk' }],
+          }),
+        }),
+      });
+
+      const mixedDSPanel = new VizPanel({
+        title: 'Panel C - Mixed',
+        pluginId: 'table',
+        key: 'panel-3',
+        $data: new SceneDataTransformer({
+          transformations: [],
+          $data: new SceneQueryRunner({
+            datasource: { uid: MIXED_DATASOURCE_NAME },
+            queries: [
+              {
+                datasource: { uid: SHARED_DASHBOARD_QUERY },
+                refId: 'A',
+                panelId: 1,
+              },
+              {
+                datasource: { uid: SHARED_DASHBOARD_QUERY },
+                refId: 'B',
+                panelId: 2,
+              },
+            ],
+            $behaviors: [new DashboardDatasourceBehaviour({})],
+          }),
+        }),
+      });
+
+      const scene = new DashboardScene({
+        title: 'hello',
+        uid: 'dash-1',
+        meta: {
+          canEdit: true,
+        },
+        body: DefaultGridLayoutManager.fromVizPanels([sourcePanel1, sourcePanel2, mixedDSPanel]),
+      });
+
+      const sceneDeactivate = activateFullSceneTree(scene);
+
+      await new Promise((r) => setTimeout(r, 1));
+
+      const spy = jest
+        .spyOn(mixedDSPanel.state.$data!.state.$data as SceneQueryRunner, 'runQueries')
+        .mockImplementation();
+
+      // First cycle: change panel 1
+      sceneDeactivate();
+      (sourcePanel1.state.$data!.state.$data as SceneQueryRunner).runQueries();
+      await new Promise((r) => setTimeout(r, 1));
+      const deactivate2 = activateFullSceneTree(scene);
+      expect(spy).toHaveBeenCalledTimes(1);
+
+      // Second cycle: change panel 2
+      deactivate2();
+      (sourcePanel2.state.$data!.state.$data as SceneQueryRunner).runQueries();
+      await new Promise((r) => setTimeout(r, 1));
+      activateFullSceneTree(scene);
+
+      // Should have been called again for panel 2
+      expect(spy).toHaveBeenCalledTimes(2);
+    });
+
+    it('Should handle multiple dashboardDS queries with library panels', async () => {
+      jest.spyOn(console, 'error').mockImplementation();
+
+      const libPanelBehavior1 = new LibraryPanelBehavior({
+        isLoaded: false,
+        uid: 'lib-panel-1',
+        name: 'Library Panel 1',
+        _loadedPanel: undefined,
+      });
+
+      const libPanelBehavior2 = new LibraryPanelBehavior({
+        isLoaded: false,
+        uid: 'lib-panel-2',
+        name: 'Library Panel 2',
+        _loadedPanel: undefined,
+      });
+
+      const sourcePanel1 = new VizPanel({
+        title: 'Panel A',
+        pluginId: 'table',
+        key: 'panel-1',
+        $behaviors: [libPanelBehavior1],
+        $data: new SceneQueryRunner({
+          datasource: { uid: 'grafana' },
+          queries: [{ refId: 'A', queryType: 'randomWalk' }],
+        }),
+      });
+
+      const sourcePanel2 = new VizPanel({
+        title: 'Panel B',
+        pluginId: 'table',
+        key: 'panel-2',
+        $behaviors: [libPanelBehavior2],
+        $data: new SceneQueryRunner({
+          datasource: { uid: 'grafana' },
+          queries: [{ refId: 'A', queryType: 'randomWalk' }],
+        }),
+      });
+
+      const mixedDSPanel = new VizPanel({
+        title: 'Panel C - Mixed',
+        pluginId: 'table',
+        key: 'panel-3',
+        $data: new SceneQueryRunner({
+          datasource: { uid: MIXED_DATASOURCE_NAME },
+          queries: [
+            {
+              datasource: { uid: SHARED_DASHBOARD_QUERY },
+              refId: 'A',
+              panelId: 1,
+            },
+            {
+              datasource: { uid: SHARED_DASHBOARD_QUERY },
+              refId: 'B',
+              panelId: 2,
+            },
+          ],
+          $behaviors: [new DashboardDatasourceBehaviour({})],
+        }),
+      });
+
+      const scene = new DashboardScene({
+        title: 'hello',
+        uid: 'dash-1',
+        meta: {
+          canEdit: true,
+        },
+        body: DefaultGridLayoutManager.fromVizPanels([sourcePanel1, sourcePanel2, mixedDSPanel]),
+      });
+
+      activateFullSceneTree(scene);
+
+      const spy = jest.spyOn(mixedDSPanel.state.$data as SceneQueryRunner, 'runQueries');
+
+      await new Promise((r) => setTimeout(r, 1));
+
+      // Should not run queries until library panels are loaded
+      expect(spy).not.toHaveBeenCalled();
+
+      // Load first library panel
+      libPanelBehavior1.setState({
+        isLoaded: true,
+        uid: 'lib-panel-1',
+        name: 'Library Panel 1',
+        _loadedPanel: undefined,
+      });
+
+      expect(spy).toHaveBeenCalledTimes(1);
+
+      // Load second library panel
+      libPanelBehavior2.setState({
+        isLoaded: true,
+        uid: 'lib-panel-2',
+        name: 'Library Panel 2',
+        _loadedPanel: undefined,
+      });
+
+      expect(spy).toHaveBeenCalledTimes(2);
+    });
+
+    it('Should handle multiple queries with transformers on all source panels', async () => {
+      jest.spyOn(console, 'error').mockImplementation();
+
+      const sourcePanel1 = new VizPanel({
+        title: 'Panel A',
+        pluginId: 'table',
+        key: 'panel-1',
+        $data: new SceneDataTransformer({
+          transformations: [{ id: 'transformA', options: {} }],
+          $data: new SceneQueryRunner({
+            datasource: { uid: 'grafana' },
+            queries: [{ refId: 'A', queryType: 'randomWalk' }],
+          }),
+        }),
+      });
+
+      const sourcePanel2 = new VizPanel({
+        title: 'Panel B',
+        pluginId: 'table',
+        key: 'panel-2',
+        $data: new SceneDataTransformer({
+          transformations: [{ id: 'transformB', options: {} }],
+          $data: new SceneQueryRunner({
+            datasource: { uid: 'grafana' },
+            queries: [{ refId: 'A', queryType: 'randomWalk' }],
+          }),
+        }),
+      });
+
+      const mixedDSPanel = new VizPanel({
+        title: 'Panel C - Mixed',
+        pluginId: 'table',
+        key: 'panel-3',
+        $data: new SceneQueryRunner({
+          datasource: { uid: MIXED_DATASOURCE_NAME },
+          queries: [
+            {
+              datasource: { uid: SHARED_DASHBOARD_QUERY },
+              refId: 'A',
+              panelId: 1,
+            },
+            {
+              datasource: { uid: SHARED_DASHBOARD_QUERY },
+              refId: 'B',
+              panelId: 2,
+            },
+          ],
+          $behaviors: [new DashboardDatasourceBehaviour({})],
+        }),
+      });
+
+      const scene = new DashboardScene({
+        title: 'hello',
+        uid: 'dash-1',
+        meta: {
+          canEdit: true,
+        },
+        body: DefaultGridLayoutManager.fromVizPanels([sourcePanel1, sourcePanel2, mixedDSPanel]),
+      });
+
+      activateFullSceneTree(scene);
+
+      await new Promise((r) => setTimeout(r, 1));
+
+      const spy = jest.spyOn(mixedDSPanel.state.$data as SceneQueryRunner, 'runQueries').mockImplementation();
+
+      // Trigger transformer reprocessing on panel 1
+      (sourcePanel1.state.$data as SceneDataTransformer).setState({
+        data: { state: LoadingState.Done, series: [], timeRange: getDefaultTimeRange() },
+      });
+
+      expect(spy).toHaveBeenCalledTimes(1);
+
+      // Trigger transformer reprocessing on panel 2
+      (sourcePanel2.state.$data as SceneDataTransformer).setState({
+        data: { state: LoadingState.Done, series: [], timeRange: getDefaultTimeRange() },
+      });
+
+      expect(spy).toHaveBeenCalledTimes(2);
+    });
   });
 
   it('Should re-run query after transformations reprocess', async () => {
+    // sometimes this tests fails with a console error `AggregateError` with an XMLHttpRequest component
+    // this is not related to the test, but a side effect of the interaction with scenes, mixed ds or even js dom
+    // considering it a flaky test, we are explicitly ignoring it by mocking console.error
+    jest.spyOn(console, 'error').mockImplementation();
     const sourcePanel = new VizPanel({
       title: 'Panel A',
       pluginId: 'table',
